/* eslint-disable */
import { isAncestor } from '@/components/popover/popoverManage'

export default class KeyboardSelect {
  constructor({
    scrollEl,
    itemElClassName = 'data-item',
    disabledClassName,
    selectIndex = -1,
    selectClassName = 'is-hover',
    topOffset,
    bottomOffset,
    handleHover = false,
    enterFn, // 按回车时的回调
    onSelectChange // hover改变时触发
  }) {
    this.itemElClassName = itemElClassName
    this.disabledClassName = disabledClassName
    this.selectClassName = selectClassName
    this.enterFn = enterFn
    this.scrollEl = getDOM(scrollEl)
    this.topOffset = topOffset
    this.bottomOffset = bottomOffset
    this.disabled = false
    this.onSelectChange = onSelectChange
    setTimeout(() => {
      this.init(selectIndex)
      if (handleHover) {
        this.scrollEl.addEventListener('mouseover', this.onMouseover)
      }
      document.addEventListener('keydown', this.onKeydown)
    })
  }
  init(selectIndex = -1) {
    setTimeout(() => {
      const itemEls = this.getEls()
      const length = itemEls.length
      if (this.topOffset === void 0 || this.bottomOffset === void 0) {
        const { pt, pb } = getPaddingAndBorderSize(this.scrollEl)
        if (this.topOffset === void 0) {
          this.topOffset = pt
        }
        if (this.bottomOffset === void 0) {
          this.bottomOffset = pb
        }
      }
      let defaultSelectIndex = typeof selectIndex === 'function' ? selectIndex(itemEls) : selectIndex
      if (!length) {
        defaultSelectIndex = -1
      } else if (length === 1) {
        defaultSelectIndex = 0
      }
      this.setSelectIndex(defaultSelectIndex)
    })
  }
  getEls() {
    return [...this.scrollEl.getElementsByClassName(this.itemElClassName)]
  }
  isAllDisabled() {
    this.getEls().every(el => el.classList.contains(this.disabledClassName))
  }
  getSelectIndex(isDown) {
    let { selectIndex } = this
    const itemEls = this.getEls()
    const length = itemEls.length
    if (this.isAllDisabled() || !length) return -1
    if (isDown) {
      do {
        if (selectIndex < length - 1) {
          selectIndex += 1
        } else {
          selectIndex = 0
        }
      } while (itemEls[selectIndex].classList.contains(this.disabledClassName))
    } else {
      do {
        if (selectIndex > 0) {
          selectIndex -= 1
        } else {
          selectIndex = length - 1
        }
      } while (selectIndex > -1 && itemEls[selectIndex].classList.contains(this.disabledClassName))
    }
    return selectIndex
  }
  setSelectIndex(selectIndex) {
    // console.log('setSelectIndex', selectIndex)
    this.getEls().forEach((el, index) => {
      el.classList[selectIndex === index ? 'add' : 'remove'](this.selectClassName)
    })
    if (this.selectIndex !== selectIndex) {
      this.onSelectChange && this.onSelectChange(selectIndex)
      this.selectIndex = selectIndex
    }
  }
  judgeSelectOptionPlacement(selectIndex) { // 判断键盘选中option处于什么位置（视图上方，视图中间，视图下方）
    if (selectIndex === -1) return
    const itemEls = this.getEls()
    const { offsetTop } = itemEls[selectIndex]
    const { scrollTop, clientHeight } = this.scrollEl
    if (scrollTop > offsetTop - this.topOffset) {
      return 'top'
    } else if (scrollTop < offsetTop - clientHeight + itemEls[selectIndex].offsetHeight + this.bottomOffset) {
      return 'bottom'
    } else {
      return 'center'
    }
  }
  onKeydown = (ev) => {
    if (this.disabled) return
    const { key } = ev
    const itemEls = this.getEls()
    if (key === 'ArrowDown' || key === 'ArrowUp') {
      const { scrollEl: el, topOffset, bottomOffset } = this
      const selectIndex = this.getSelectIndex(key === 'ArrowDown')
      this.setSelectIndex(selectIndex)
      if (selectIndex === -1) return
      const placement = this.judgeSelectOptionPlacement(selectIndex)
      if (placement === 'top') {
        el.scrollTo({
          top: selectIndex === 0 ? 0: itemEls[selectIndex].offsetTop - topOffset,
          behavior: 'smooth',
        })
      } else if (placement === 'bottom') {
        el.scrollTo({
          top: itemEls[selectIndex].offsetTop - el.clientHeight + itemEls[selectIndex].offsetHeight + bottomOffset,
          behavior: 'smooth',
        })
      }
      ev.preventDefault && ev.preventDefault()
    } else if (key === 'Enter') {
      if (this.selectIndex > -1 && itemEls.length && this.enterFn) {
        this.enterFn(itemEls[this.selectIndex], this.selectIndex)
      }
    }
  }
  onMouseover = (e) => {
    const el = isAncestor(e.target, this.itemElClassName)
    if (el) {
      const index = this.getEls().findIndex(e => e === el)
      this.setSelectIndex(index)
    }
  }
  clear() {
    document.removeEventListener('keydown', this.onKeydown)
    this.scrollEl.removeEventListener('mouseover', this.onMouseover)
  }
}

const isDOM = typeof HTMLElement === 'object'
  ? function (obj) {
    return obj instanceof HTMLElement
  }
  : function (obj) {
    return obj && typeof obj === 'object' && obj.nodeType === 1 && typeof obj.nodeName === 'string'
  }
export function getPaddingAndBorderSize(el) {
  const styleObj = getComputedStyle(el)
  const pl = parseInt(styleObj.paddingLeft)
  const pr = parseInt(styleObj.paddingRight)
  const pt = parseInt(styleObj.paddingTop)
  const pb = parseInt(styleObj.paddingBottom)

  const bl = parseInt(styleObj.borderLeftWidth)
  const br = parseInt(styleObj.borderRightWidth)
  const bt = parseInt(styleObj.borderTopWidth)
  const bb = parseInt(styleObj.borderBottomWidth)

  return {
    pl,
    pr,
    pt,
    pb,
    bl,
    br,
    bt,
    bb,
    bs: styleObj.boxSizing,
    px: pl + pr,
    py: pt + pb,
    bx: bl + br,
    by: bt + bb,
    x: pl + pr + bl + br,
    y: pt + pb + bt + bb
  }
}
function getDOM(elOrSelector) {
  if (isDOM(elOrSelector)) return elOrSelector
  if (typeof elOrSelector === 'string') {
    return document.querySelector(elOrSelector)
  }
}

