import LoadingIcon from '@/icon/icon-loading.png'
import './index.less'
const loadingTransitionTime = 200

function createLoadingElement(ref, modifiers, style) {
  const el = document.createElement('div')
  const type = ['scale', 'rotate', 'jump'].find(t => modifiers[t]) || 'rotate'
  el.className = `custom-loading-el loading loading-${type}`
  let bgc
  bgc = 'rgb(255, 255, 255, 0.6)'
  el.style.backgroundColor = bgc
  el.style.borderRadius = getComputedStyle(ref).borderRadius
  if (style) {
    el.style.cssText += style
  }
  const indicatorWrap = document.createElement('div')
  indicatorWrap.className = 'custom-loading-indicator-wrap'
  if (type === 'scale') {
    for (let i = 0; i < 5; i++) {
      const span = document.createElement('span')
      span.className = 'custom-loading-indicator-el'
      indicatorWrap.appendChild(span)
    }
  } else if (type === 'rotate') {
    const i = document.createElement('img')
    i.src = LoadingIcon
    i.className = 'loading-indicator-icon'
    indicatorWrap.appendChild(i)
  } else if (type === 'jump') {
    for (let i = 0; i < 3; i++) {
      const span = document.createElement('span')
      span.className = 'loading-indicator-icon'
      indicatorWrap.appendChild(span)
    }
  }
  el.appendChild(indicatorWrap)
  ref.appendChild(el)
  showLoading(el, modifiers)
}
async function showLoading(loadingEl, { center, noTransitionStart }) {
  if (noTransitionStart) {
    loadingEl.classList.add('loading')
  } else {
    loadingEl.classList.add('loading', 'opacity0')
    await new Promise(resolve => setTimeout(resolve, 0))
  }
  loadingEl.classList.remove('opacity0')
  if (!center && loadingEl.offsetParent) {
    const indicatorWrap = loadingEl.querySelector('.custom-loading-indicator-wrap')
    const { offsetTop, offsetLeft } = indicatorWrap
    indicatorWrap.style.position = 'absolute'
    indicatorWrap.style.left = offsetLeft + 'px'
    indicatorWrap.style.top = offsetTop + 'px'
  }
}
function findLoadingElement(target) {
  for (let i = target.children.length - 1; i >= 0; i--) {
    const e = target.children[i]
    if (e && e.classList.contains('custom-loading-el')) {
      return e
    }
  }
}
function resolveValue(el, value) {
  if (typeof value === 'object' && value) {
    return {
      value: value.value,
      el: value.target
        ? typeof value.target === 'function'
          ? value.target(el)
          : el.querySelector(value.target)
        : el,
      style: value.style || ''
    }
  }
  return { el, value }
}

export default {
  mounted(el, { value, modifiers }) {
    let style
    ;({ el, value, style } = resolveValue(el, value))
    const { position } = getComputedStyle(el)
    if (position === 'static') {
      el.style.position = 'relative'
    }
    if (value) {
      createLoadingElement(el, modifiers, style)
    }
  },
  updated(el, { value, oldValue, modifiers }) {
    let style
    if (typeof value === 'object') {
      ;({ el, value, style } = resolveValue(el, value))
      oldValue = oldValue.value
    }
    if (value === oldValue) return
    const loadingElement = findLoadingElement(el)
    if (value) { // 显示loading
      if (loadingElement) {
        showLoading(loadingElement, modifiers)
      } else {
        createLoadingElement(el, modifiers, style)
      }
    } else if (loadingElement) { // 取消显示
      setTimeout(() => {
        loadingElement.classList.remove('loading', 'opacity0')
        const indicatorWrap = loadingElement.getElementsByClassName('custom-loading-indicator-wrap')[0]
        indicatorWrap && indicatorWrap.style.removeProperty('position')
      }, loadingTransitionTime)
      loadingElement.classList.add('opacity0')
    }
  },
  beforeUnmount(el, { value }) {
    ;({ el } = resolveValue(el, value))
    const loadingElement = findLoadingElement(el)
    loadingElement && loadingElement.remove()
  }
}
