import { ref } from 'vue'
import { api_showDepartment, api_showDepartmentMember, api_showUserGroup } from '@/api'
import { formatDataItem } from '@/utils'
import { traverseRecursively } from '@/tools'
export function formatDepartments(list) {
	traverseRecursively(
		list,
		(val, parent) => {
			Object.assign(val, {
				...formatDataItem.department(val),
				$path: parent ? [...parent.$path, val] : [val]
			})
		},
		'$children',
		null,
	)
	return list
}

export function formatUsers(list) {
	return list.map(user => {
		return formatDataItem.user(user)
	})
}

export function formatUserGroups(list) {
	traverseRecursively(
		list,
		(val, parent) => {
			Object.assign(val, {
				...formatDataItem.userGroup(val),
				$path: parent ? [...parent.$path, val] : [val]
			})
		},
		'$children',
		null
	)
	return list
}
export function useOrganizationData() {
	const departments = ref([])
	const users = ref([])
	const userGroups = ref([])
	function getDepartments(params) {
		return api_showDepartment(params).then(res => {
			departments.value = formatDepartments([res.data])
		})
	}
	function getUsers(params) {
		return api_showDepartmentMember(params).then(res => {
			users.value = formatUsers(res.data)
		})
	}
	function getUserGroups(params) {
		return api_showUserGroup(params).then(res => {
			userGroups.value = formatUserGroups(res.data)
		})
	}
	return {
		departments, users, userGroups, getUsers,
		getUserGroups, getDepartments,
	}
}
