import { ref, computed } from 'vue'
import { useOrganizationData } from '@/hooks/useOrganizationData'
import { getDepartmentsMap, getUserMap, getUserGroupsMap } from '@/utils'
export function useOrganizationStore() {
	const selected = ref([])
	const { departments, users, userGroups, getDepartments, getUsers, getUserGroups, } = useOrganizationData()
	const departmentMap = computed(() => getDepartmentsMap(departments.value))

	const userGroupMap = computed(() => getUserGroupsMap(userGroups.value))

	const departmentUsers = computed(() => {
		return users.value.reduce((result, user) => {
			user.$department.forEach(dpt => {
				departmentMap.value[dpt.department_id]?.$path?.forEach(dpt => {
					result[dpt.department_id] = result[dpt.department_id] ? [...result[dpt.department_id]] : []
					!result[dpt.department_id].some(_user => user.$id === _user.$id) && result[dpt.department_id].push(user)
				})
			})
			return result
		}, {})
	})
	const userGroupUsers = computed(() => {
		return users.value.reduce((result, user) => {
			user.$userGroup.forEach(dpt => {
				result[dpt.department_id] = result[dpt.department_id] ? [...result[dpt.department_id], user] : [user]
			})
			return result
		}, {})
	})

	const userMap = computed(() => getUserMap(users.value))
	return {
		selected, departments, departmentMap, users, userGroups,
		userGroupMap, userMap, getDepartments, getUsers,
		getUserGroups, departmentUsers, userGroupUsers,
	}
}
