import { computed, ref, unref, watch, h, Fragment } from 'vue'
import { api_search } from '@/api'
import { formatDataItem, searchTab, tabsMap } from '@/utils'
import { debounce } from '@/tools'

export function useSearch({ tabs, props, searchDelay = 200 }) {
	const keywords = ref('')
	const searchResult = ref([])
	const loading = ref(false)
	const isEmpty = computed(() => !loading.value && keywords.value && searchResult.value.every(item => !item.resultList?.length))
	watch(keywords, val => {
		!val && (searchResult.value.length = 0)
	})
	function onSearch() {
		if (!keywords.value) {
			searchResult.value.length = 0
			loading.value = false
			return
		}
		loading.value = true
		const tabsValue = unref(tabs) || []
		const params = {
			keywords: keywords.value,
			is_pass: 1,
			...tabsValue.reduce((tabsParams, t) => Object.assign(tabsParams, { [t.apiValue]: '1' }), {})
		}
		if (props.userId) {
			params.user_uuid = props.userId
		}
		api_search(params).then(res => {
			searchResult.value.length = 0
			const result = []
			searchTab.filter(t => tabsValue.some(item => item.value === t.value)).forEach(tab => {
				result.push({
					...tab,
					resultList: (res.data[tab.apiValue] || []).map(item => {
						const formattedData = formatDataItem[tab.value](item)
						return {
							...formattedData,
							$icon: tab.activeIcon,
							$label: formattedData.$type === 'department' ? item.full_department :
								formattedData.$type === 'userGroup' ? item.type_group :
									formattedData.$type === 'user' ? () => h(Fragment, null, [
											h('span', {}, formattedData.$name),
											formattedData.department_major?.department ?
												h('span', {
														style: {
															color: '#999',
															fontWeight: 'normal',
															fontSize: '12px'
														}
													},  `（${formattedData.department_major.department}）`) :
												null
										]) :
									formattedData.$name,
							$value: formatDataItem[tab.value](item),
						}}),
				})
				searchResult.value = result
			})
		}).finally(_ => {
			loading.value = false
		})
	}
	const searchDebounce = debounce(onSearch, searchDelay)
	return {
		keywords, onSearch: () => {
			loading.value = true
			searchDebounce()
		}, searchResult,
		loading, isEmpty,
	}
}
