import { useOrganizationStore } from '@/hooks/useOrganizationStore'
import { provide } from 'vue'
import { useCommonGroups } from '@/hooks/useCommonGroups'
export function useStore({ emit, props }) {
	const {
		selected, departments, departmentMap, userGroupMap,
		users, userGroups, userMap, getDepartments, getUsers,
		departmentUsers, userGroupUsers,
		getUserGroups,
	} = useOrganizationStore()
	const { getCommonGroups } = useCommonGroups()

	const store = {
		departments, users, userGroups, departmentMap,
		userGroupMap, userMap, departmentUsers, userGroupUsers,
	}
	provide('selected', selected)
	provide('updateSelected', updateSelected)
	provide('store', store)
	function updateSelected(val, shouldEmit = true) {
		selected.value = val
		shouldEmit && emit('change', [...val])
	}
	function updateStore(type) {
		const params = props.userId ? { user_uuid: props.userId } : {}
		let result
		switch (type) {
			case 'department':
				result = getDepartments(params)
				break
			case 'user':
				result = getUsers(params)
				break
			case 'userGroup':
				result = getUserGroups(params)
				break
			case 'commonGroup':
				result = getCommonGroups(params)
				break
			default:
				result = Promise.all([
					getDepartments(params), getUsers(params), getUserGroups(params),
					getCommonGroups(params)
				])
		}
		return result
	}
	provide('updateStore', updateStore)
	return {
		selected, store, updateStore, updateSelected,
	}
}
