import { fileURLToPath, URL } from 'node:url'
import { resolve } from 'path'

import { defineConfig } from 'vite'
import vue from '@vitejs/plugin-vue'
import Components from 'unplugin-vue-components/vite'
import { VantResolver } from '@vant/auto-import-resolver'
import libCss from 'vite-plugin-libcss'

export default defineConfig({
  plugins: [
    Components({
      resolvers: [VantResolver()],
    }),
    vue(),
    libCss(),
  ],
  resolve: {
    alias: {
      '@': fileURLToPath(new URL('../../lib', import.meta.url))
    }
  },
  css: {
    preprocessorOptions: {
      less: {
        sourceMap: false,
        javascriptEnabled: true,
      }
    }
  },
  build: {
    target: 'es2015',
    lib: {
      entry: resolve(__dirname, './index.js'),
      name: 'OneOrganizationSelect',
      fileName: 'index',
      formats: ['es', 'cjs'],
      include: ['**/node_modules/**'],
    },
    outDir: 'mobile',
    rollupOptions: {
      external: ['vue', 'axios', 'js-md5', 'vant'],
    },
    cssCodeSplit: true,
  },
})
