
// 递归遍历树状节点
export function traverseRecursively(list, cb, listKey = 'list', parentNode) {
	if (!list || !list.length) return
	for (let i = 0, len = list.length; i < len; i++) {
		const item = list[i]
		cb(item, parentNode)
		if (item[listKey] && item[listKey].length) {
			traverseRecursively(item[listKey], cb, listKey, item)
		}
	}
}
export function debounce(fn, delay = 300, async = false, context) {
	let timer
	if (async) {
		return async function (...args) {
			clearTimeout(timer)
			return new Promise((resolve, reject) => {
				timer = setTimeout(async () => {
					try {
						const res = await fn.apply(context || this, args)
						resolve(res)
					} catch (e) {
						reject(e)
					}
				}, delay)
			})
		}
	} else {
		return function (...args) {
			clearTimeout(timer)
			timer = setTimeout(() => {
				fn.apply(context || this, args)
			}, delay)
		}
	}
}


// 获取字符串的字节长度，1个汉字两个字节，英文数字一个字节，繁体三个字节
export function getByteLength(str = '') {
	let len = 0
	for (let i = 0; i < str.length; i++) {
		if (str.charCodeAt(i) > 127 || str.charCodeAt(i) === 94) {
			len += 2
		} else {
			len++
		}
	}
	return len
}

export function sliceByByteLength(str, length) {
	let result = '',
		strlen = str.length,
		byteLen = getByteLength(str)

	if (byteLen <= length) return str

	for (let i = 0, j = 0; i < strlen; i++) {
		let chr = str.charAt(i)
		if (/[\x00-\xff]/.test(chr)) {
			j++
		} else {
			j += 2
		}
		if (j <= length) {
			result += chr
		} else {
			return result
		}
	}
}

export function fromPairs(pairs) {
	const result = {}
	if (pairs == null) {
		return result
	}
	for (const pair of pairs) {
		result[pair[0]] = pair[1]
	}
	return result
}
export const isDOM = typeof HTMLElement === 'object'
	? function (obj) {
		return obj instanceof HTMLElement
	}
	: function (obj) {
		return obj && typeof obj === 'object' && obj.nodeType === 1 && typeof obj.nodeName === 'string'
	}

export function assignWithProto(targetObj, originObj) {
	for (const key in originObj) {
		targetObj[key] = originObj[key]
	}
	return targetObj
}

export const delay = (delayMs) => new Promise(resolve => setTimeout(resolve, delayMs))

let seed = 0
export function generateId() {
	return '' + Date.now() + seed++
}

export function throttle(fn, delay = 300, context) {
	let canDo = true
	return function (...args) {
		if (!canDo) return
		fn.apply(context || this, args)
		canDo = false
		setTimeout(() => {
			canDo = true
		}, delay)
	}
}
