import { unref } from 'vue'
import { traverseRecursively } from '@/tools'

export const organizationType = {
	department: {
		name: '部门',
		value: 'department',
		apiValue: 'department',
	},
	userGroup: {
		name: '用户组',
		value: 'userGroup',
		apiValue: 'group',
	},
	user: {
		name: '成员',
		value: 'user',
		apiValue: 'member',
	}
}
export const organizationIcon = {
	department: {
		icon: new URL('@/icon/icon-department.png', import.meta.url).href,
		activeIcon: new URL('@/icon/icon-department-active.png', import.meta.url).href,
	},
	userGroup: {
		icon: new URL('@/icon/icon-group.png', import.meta.url).href,
		activeIcon: new URL('@/icon/icon-group-active.png', import.meta.url).href,
	},
	user: {
		icon: new URL('@/icon/icon-user.png', import.meta.url).href,
		activeIcon: new URL('@/icon/icon-user-active.png', import.meta.url).href,
	},
	commonGroup: {
		icon: new URL('@/icon/icon-common-group.png', import.meta.url).href,
		activeIcon: new URL('@/icon/icon-common-group-active.png', import.meta.url).href,
	}

}
export const tabs = [
	{
		...organizationType.department,
		...organizationIcon.department
	},
	{
		...organizationType.userGroup,
		...organizationIcon.userGroup,
	},
	{
		...organizationType.user,
		...organizationIcon.user,
	},
]
export const searchTab = [
	tabs[2], tabs[0], tabs[1]
]
export const tabsMap = tabs.reduce((result, curr) => {
	return Object.assign(result, { [curr.value]: curr})
}, {})

export const organizationsColor = {
	department: {
		color: '#33AEEE',
		background: '#E8F7FF',
	},
	userGroup: {
		color: '#FD8243',
		background: '#FFEFE8',
	},
	user: {
		color: '#9B64F3',
		background: '#F1ECF9',
	},
}

export const formatDataItem = {
	department: data => {
		const { department, department_id, list = [] } = data
		return {
			$name: department || '', $id: department_id || '',
			$type: 'department',
			$children: list.map(item => ({ ...item })),
			...data,
		}
	},
	userGroup: data => {
		const {
			user_group_name, user_group_uuid, type, type_uuid,
			user_group = [],
		} = data
		return {
			$name: user_group_name || type, $id: user_group_uuid || type_uuid,
			$type: 'userGroup',
			$groupType: user_group_uuid ? 'userGroup' : 'userGroupType',
			$children: user_group.map(item => ({ ...item })),
			...data,
		}
	},
	user: data => {
		const {
			nickname, user_uuid, department = [],
			user_group = [],
		} = data
		return {
			$name: nickname, $id: user_uuid,
			$type: 'user',
			$department: department.map(item =>  formatDataItem.department(item)),
			$userGroup: user_group.map(item => formatDataItem.userGroup(item)),
			...data,
		}
	},
	commonGroup: data => {
		const {
			department, department_id, list = [],
		} = data
		return {
			$name: department,
			$id: department_id,
			$type: 'commonGroup',
			$users: list.map(item => formatDataItem.user(item)),
			...data,
		}
	},
}

export const formatExposeDataItem = data => {
	const dataAttr = Object.keys(data).reduce(
		(res, key) => {
			if (!key.startsWith('$')) {
				Object.assign(res, {
					[key]: unref(data[key])
				})
			}
			return res
		},
		{}
	)
	return {
		type: data.$type,
		value: data.$id,
		data: {
			...dataAttr,
			$store: unref(data),
		},
	}
}

export function formatDisabledProps(value = []) {
	return value.map(item => {
		return {
			$id: item.value,
			$type: item.type
		}
	})
}

export function getDepartmentsMap(departments) {
	const result = {}
	traverseRecursively(
		departments,
		(val) => {
			Object.assign(result, { [val.$id]: val })
		},
		'$children',
	)
	return result
}

export function getUserGroupsMap(userGroups) {
	const result = {}
	userGroups.forEach(item => {
		item.$children.forEach(val => {
			Object.assign(result, { [val.$id]: val })
		})
	})
	return result
}

export function getUserMap(users) {
	return users.reduce((result, user) => Object.assign(result, { [user.$id]: user }), {})
}
