import { shallowRef, markRaw } from 'vue'
import { ElMessageBox } from 'element-plus'
import { Delete } from '@element-plus/icons-vue'
import { getByteLength } from '@/tools'
import { api_addPersonalDep, api_delPersonalDep, api_opeDepName } from '@/api'

export function useCommonHandler(cb = () => null) {
	const menus = shallowRef([
		{ label: '重命名', handler: onRename, },
		{ label: '删除', handler: onDelete, },
	])
	function onDelete(data) {
		ElMessageBox.confirm(
			`是否删除常用组『${data.$name}』?`,
			'',
			{
				type: 'warning',
				icon: markRaw(Delete),
				confirmButtonText: '确认',
				cancelButtonText: '取消',
			}
		).then(_ => {
			api_delPersonalDep({
				department_id: data.$id,
			}).then(_ => {
				cb()
			})
		})
	}
	function onRename(data) {
		ElMessageBox.prompt('输入常用组名称', '重命名', {
			confirmButtonText: '确认',
			cancelButtonText: '取消',
			inputValidator: val => {
				if (getByteLength(val) > 30) return '超出字符限制'
				if (!val) return '请输入名称'
				return true
			},
			inputPlaceholder: '输入常用组名称',
			inputValue: data.$name,
		})
			.then(({ value }) => {
				api_opeDepName({
					department_id: data.$id,
					department: value,
				}).then(_ => {
					data.$name = value
					cb()
				})
			})

	}
	function onNew(users) {
		ElMessageBox.prompt('输入常用组名称', '保存为常用组', {
			confirmButtonText: '确认',
			cancelButtonText: '取消',
			inputValidator: val => {
				if (getByteLength(val || '') > 30) return '超出字符限制'
				if (!val) return '请输入名称'
				return true
			},
			inputPlaceholder: '输入常用组名称',
		})
			.then(({ value }) => {
				api_addPersonalDep({
					department: value,
					member: users.map(u => u.user_uuid).join('|')
				}).then(_ => {
					cb()
				})
			})

	}
	return {
		menus, onDeleteCommonGroup: onDelete,
		onRenameCommonGroup: onRename, onNewCommonGroup: onNew,
	}
}
