import { computed, ref } from 'vue'
import { api_showAllDepartment, api_showAllDepartmentMember, api_showUserGroup } from '@/api'
import { formatDepartments, formatUsers, formatUserGroups } from '@/hooks/useOrganizationData'
import { getDepartmentsMap, getUserGroupsMap, getUserMap } from '@/utils'

const departments = ref([])
const users = ref([])
const userGroups = ref([])
const departmentMap = computed(() => getDepartmentsMap(departments.value))

const userGroupMap = computed(() => getUserGroupsMap(userGroups.value))
const userMap = computed(() => getUserMap(users.value))
const departmentUsers = computed(() => {
	return users.value.reduce((result, user) => {
		user.$department.forEach(dpt => {
			departmentMap.value[dpt.department_id]?.$path?.forEach(dpt => {
				result[dpt.department_id] = result[dpt.department_id] ? [...result[dpt.department_id]] : []
				!result[dpt.department_id].some(_user => user.$id === _user.$id) && result[dpt.department_id].push(user)
			})
		})
		return result
	}, {})
})
const userGroupUsers = computed(() => {
	return users.value.reduce((result, user) => {
		user.$userGroup.forEach(dpt => {
			result[dpt.department_id] = result[dpt.department_id] ? [...result[dpt.department_id], user] : [user]
		})
		return result
	}, {})
})

const EXPIRES = 20000
let lastUpdate = 0
let updateRequest = Promise.resolve()
export function useOrganizationData() {
	function fetchDepartments() {
		return api_showAllDepartment().then(res => {
			departments.value = formatDepartments([res.data])
		})
	}
	function fetchUsers() {
		return api_showAllDepartmentMember().then(res => {
			users.value = formatUsers(res.data)
		})
	}
	function fetchUserGroups() {
		return api_showUserGroup().then(res => {
			userGroups.value = formatUserGroups(res.data)
		})
	}
	function updateOrganizationData() {
		if (Date.now() > (lastUpdate + EXPIRES)) {
			lastUpdate = Date.now()
			return updateRequest = Promise.all([
				fetchDepartments(),
				fetchUsers(),
				fetchUserGroups()
			])
		} else {
			return updateRequest
		}
	}
	const userStore = {
		departmentUsers: computed(() => {
			return users.value.reduce((result, user) => {
				user.$department.forEach(dpt => {
					departmentMap.value[dpt.department_id]?.$path?.forEach(dpt => {
						result[dpt.department_id] = result[dpt.department_id] ? [...result[dpt.department_id]] : []
						!result[dpt.department_id].some(_user => user.$id === _user.$id) && result[dpt.department_id].push(user)
					})
				})
				return result
			}, {})
		}),
		userGroupUsers: computed(() => {
			return users.value.reduce((result, user) => {
				user.$userGroup.forEach(dpt => {
					result[dpt.department_id] = result[dpt.department_id] ? [...result[dpt.department_id], user] : [user]
				})
				return result
			}, {})
		}),
	}
	const store = {
		departments, users, userGroups, departmentMap,
		userGroupMap, userMap, departmentUsers, userGroupUsers,
	}
	return {
		departments, users, userGroups, departmentMap,
		userMap, userGroupMap, departmentUsers, userGroupUsers,
		updateOrganizationData, userStore, store,
	}
}
