import { fileURLToPath, URL } from 'node:url'
import { resolve } from 'path'

import { defineConfig } from 'vite'
import vue from '@vitejs/plugin-vue'
import AutoImport from 'unplugin-auto-import/vite'
import Components from 'unplugin-vue-components/vite'
import { ElementPlusResolver } from 'unplugin-vue-components/resolvers'
import libCss from 'vite-plugin-libcss'

export default defineConfig({
  plugins: [
    AutoImport({
      resolvers: [ElementPlusResolver()],
    }),
    // Components({
    //   resolvers: [ElementPlusResolver()],
    // }),
    // libCss(),
    vue(),
  ],
  resolve: {
    alias: {
      '@': fileURLToPath(new URL('../../lib', import.meta.url))
    }
  },
  css: {
    preprocessorOptions: {
      less: {
        sourceMap: false,
        javascriptEnabled: true,
      }
    }
  },
  define: {
    'process.env.NODE_ENV': '"production"',
  },
  build: {
    target: 'es2015',
    lib: {
      entry: resolve(__dirname, './index.js'),
      name: 'OneOrganizationSelect',
      fileName: 'index',
    },
    outDir: 'web',
    rollupOptions: {
      external: ['vue', 'axios', 'js-md5', 'element-plus', '@element-plus/icons-vue', 'qtable-auth-hook'],
    },
    cssCodeSplit: true,
  },
})
